clear all; close all
%--------------------------------------------------------------------------
%   Main program for applying the processing of passive
%   seismic data (spectral whitening and auto/cross-correlation)
%
%   1. The seismic data (vertical, radial comp) are provided as SAC files
%       (use readsac.m to read SAC files)
%
%   2. The seismograms are cut in windows surrounding the P-wave arrival
%   (The arrival time of P-wave is in the fields z.t0, r.t0)
%
%   3. Safer to apply a taper (here standard matlab function tukeywin.m)
%
%   4. Apply deconvolution (spectral whitening and auto/cross-correlation)
%       with matlab function. 
%       ** Care ** 
%       Portion of signal before P-wave fixed to t_delay = 20 s
%       in apply_spectral_whitening_deconvolution.m
%
%
% Benoit TAUZIN (PhD)
% benoit.tauzin@univ-lyon1.fr / benoit.tauzin@anu.edu.au
%
% Research School of Earth Science
% Australian National University, Canberra, Australia
%
% Laboratoire de Geologie de Lyon
% Universite de Lyon, Villeurbanne, France
%
% last updated 12.02.2019
% ------------------------------------------------------------------------

%-- PARAMETERS ----------------------------------------------------------
Twin=[15 100];                      % WINDOWING: TIME BEFORE AND AFTER P-
a_large_gauss=2.5;                  % CUT-OFF FOR GAUSSIAN LOW-PASS FILTER
W=.1;                              % SPECTRAL WIDTH FOR SMOOTHING

%% READ AND WINDOW TRACES
% READ VERTICAL AND RADIAL COMPS (SAC FILES)
file_name='./2008.09.08-18.52.07.ME25.01.Z.0-0.SAC';
z=readsac(file_name);
file_name='./2008.09.08-18.52.07.ME25.01.R.0-1.SAC';
r=readsac(file_name);
% WINDOW VERTICAL AND RADIAL COMPONENTS
i1=floor((z.t0-Twin(1))/z.delta); i2=i1+floor(Twin(2)/z.delta)+1;
z.trace=z.trace(i1:i2); z.npts=length(z.trace);
r.trace=r.trace(i1:i2); r.npts=length(r.trace);
z.t0=Twin(1); r.t0=Twin(1);
% APPLY A TAPER
wt=tukeywin(z.npts,.2);
z.trace=z.trace.*wt; r.trace=r.trace.*wt;

%% SPECTRAL WHITENING DECONVOLUTION
[rfz,rf]=apply_spectral_whitening_deconvolution(z,r,W,a_large_gauss);

%% PLOT
h1.hf=figure(1);
h1.ha1=subplot(2,1,1);
pos=get(h1.ha1,'Position');
set(h1.ha1,'box','on','TickDir','out',...
    'XTick',[-20:10:80],'YTick',[0 0.4],'YTickLabel','');
hold on; grid on
plot([0:rfz.npts-1]*rfz.delta-rfz.o,rfz.trace+.4,'k','LineWidth',1.5)
plot([0:rf.npts-1]*rf.delta-rf.o,rf.trace,'k','LineWidth',1)
axis([-20 80 -.2 .7])
xlabel('time (s)')
legend('vertical auto-correlation (vertical receiver function)','cross-correlation (radial receiver function)', 'Location','NorthOutside')
